#%%
import numpy as np
import matplotlib.pyplot as plt
import importlib
import audio
import warnings
warnings.filterwarnings('ignore')

importlib.reload(audio)
from audio import MicrophoneArray


# create linear array class (l_total = 0.14m, num_mic=5, )
linear_array = MicrophoneArray()

# load audio: run tests with 12 voice audio files, 4 music files
linear_array.load_audio("audio_data/MU1_10000000_01_S01.wav")
linear_array.load_audio("audio_data/MU1_10000000_02_S01.wav")
linear_array.load_audio("audio_data/MU1_10000000_03_S01.wav")
linear_array.load_audio("audio_data/MU1_10000300_01_S02.wav")
linear_array.load_audio("audio_data/MU1_10000300_02_S02.wav")
linear_array.load_audio("audio_data/MU1_10000300_03_S02.wav")
linear_array.load_audio("audio_data/MU1_10000600_01_S03.wav")
linear_array.load_audio("audio_data/MU1_10000600_02_S03.wav")
linear_array.load_audio("audio_data/MU1_10000600_03_S03.wav")
linear_array.load_audio("audio_data/MU1_10000900_01_S04.wav")
linear_array.load_audio("audio_data/MU1_10000900_02_S04.wav")
linear_array.load_audio("audio_data/MU1_10000900_03_S04.wav")

linear_array.load_audio("audio_data/MU1_10000000_01___M.wav", type='m')
linear_array.load_audio("audio_data/MU1_10000000_02___M.wav", type='m')
linear_array.load_audio("audio_data/MU1_10000000_03___M.wav", type='m')
linear_array.load_audio("audio_data/MU1_10000000_03___M.wav", type='m')

# define number of trials for each angle and all angles to test
n_exp = 30
theta_list = np.linspace(30, 0, 90)

#%% DEFINE EXPERIMENT
## First Scenario (2 voices, music noise)
labels = ['Voice 1', 'Voice 2', 'Music']
n_voices = 2

# SCENARIO 1 EXAMPLE PLOT 1
theta = 20
v1_theta = np.deg2rad(theta/2) # direction of steering 
v2_theta = np.deg2rad(-theta/2) # direction of incident sound 
m_theta = 0
angles = [v1_theta, v2_theta, m_theta]

results = np.zeros((2, len(angles)))
for each in np.arange(n_exp):
    # randomize experiment 
    sounds = linear_array.randomize_experiment(n_voices,(len(angles)-n_voices))
    # combine sounds based on acoustic model, outputs frequency domain signal at each microphone
    r = linear_array.combine_freq_domain(incoming_angles=angles, signals=sounds)
    # add noise
    r = linear_array.add_white_noise(r)

# plot the resulting PSNR across steering angle
linear_array.listen_pattern_fixed_input(r, sounds, labels, model=MicrophoneArray.w_mvdr)


#%%
# save audio for demos
output = linear_array.listen(m_theta, r, model=MicrophoneArray.w_mvdr)
linear_array.load_write(output, f"audio_output/mvdr_m1_{n_voices}_{theta}.wav")
output = linear_array.listen(v1_theta, r, model=MicrophoneArray.w_mvdr)
linear_array.load_write(output, f"audio_output/mvdr_v1_{n_voices}_{theta}.wav")
output = linear_array.listen(v2_theta, r, model=MicrophoneArray.w_mvdr)
linear_array.load_write(output, f"audio_output/mvdr_v2_{n_voices}_{theta}.wav")

#%%
# SCENARIO 1 EXAMPLE PLOT 2
theta = 5
v1_theta = np.deg2rad(theta/2) # direction of steering 
v2_theta = np.deg2rad(-theta/2) # direction of incident sound 
m_theta = 0
angles = [v1_theta, v2_theta, m_theta]

results = np.zeros((2, len(angles)))
for each in np.arange(n_exp):
    # randomize experiment 
    # combine sounds based on acoustic model, outputs frequency domain signal at each microphone
    # add noise
    sounds = linear_array.randomize_experiment(n_voices,(len(angles)-n_voices))
    r = linear_array.combine_freq_domain(incoming_angles=angles, signals=sounds)
    r = linear_array.add_white_noise(r)

# plot the result
linear_array.listen_pattern_fixed_input(r, sounds, labels, model=MicrophoneArray.w_mvdr)

# save audio for demos
output = linear_array.listen(m_theta, r, model=MicrophoneArray.w_mvdr)
linear_array.load_write(output, f"audio_output/mvdr_m1_{n_voices}_{theta}.wav")
output = linear_array.listen(v1_theta, r, model=MicrophoneArray.w_mvdr)
linear_array.load_write(output, f"audio_output/mvdr_v1_{n_voices}_{theta}.wav")
output = linear_array.listen(v2_theta, r, model=MicrophoneArray.w_mvdr)
linear_array.load_write(output, f"audio_output/mvdr_v2_{n_voices}_{theta}.wav")


#%%

## SCENARIO 1 PLOT OVER THETA
aug = []
sup = []

for theta in theta_list:
    print(theta)
    v1_theta = np.deg2rad(theta/2) # direction of steering 
    v2_theta = np.deg2rad(-theta/2) # direction of incident sound 
    m_theta = 0
    angles = [v1_theta, v2_theta, m_theta]

    results = np.zeros((2, len(angles)))
    for each in np.arange(n_exp):
        # randomize experiment 
        # combine sounds based on acoustic model, outputs frequency domain signal at each microphone
        # add noise
        sounds = linear_array.randomize_experiment(n_voices,(len(angles)-n_voices))
        r = linear_array.combine_freq_domain(incoming_angles=angles, signals=sounds)
        r = linear_array.add_white_noise(r)
        
        # record the percent improvment for eye-tracking
        results += linear_array.listen_improvement(r, sounds, angles, model=MicrophoneArray.w_mvdr)

        # if all plots are desired
        # linear_array.listen_pattern_fixed_input(r, sounds, labels, model=MicrophoneArray.w_mvdr)
    results = results/n_exp

    avg_augmentation = (results[0,0] + results[0,1])/2
    avg_supression = (results[1,0] + results[1,1])/2
    aug.append(avg_augmentation)
    sup.append(avg_supression)

# plot percent improvment
fig, ax = plt.subplots(1, 1, figsize=(7,3))
ax.set_title("Change in PSNR for Eye-Steered versus Head-Steered")
ax.set_ylabel("Percent (%) Change in PSNR")
ax.set_xlabel("Angular Distance Between Sources (deg)")
ax.plot(theta_list[:-1], aug[:-1], label='PSNR improvement in target source') 
ax.plot(theta_list[:-1], sup[:-1], label='PSNR suppression in interference source') 
ax.invert_xaxis()
ax.legend()
ax.set_ylim([0,1])
print(aug)
print(sup)

#%%

## Second Scenario (4 voices, music noise)
labels = ['Voice 1', 'Voice 2', 'Voice 3', 'Voice 4', 'Music']
n_voices = 4

# SCENARIO 2 EXAMPLE PLOT 1
theta = 20
v1_theta = np.deg2rad(theta/2) 
v2_theta = np.deg2rad(-theta/2) 
v3_theta = np.deg2rad(theta/2 + theta) 
v4_theta = np.deg2rad(-theta/2 - theta) 
m_theta = 0
angles = [v1_theta, v2_theta, v3_theta, v4_theta, m_theta]

results = np.zeros((2, len(angles)))
for each in np.arange(n_exp):
    # randomize experiment 
    # combine sounds based on acoustic model, outputs frequency domain signal at each microphone
    # add noise
    sounds = linear_array.randomize_experiment(n_voices,(len(angles)-n_voices))
    r = linear_array.combine_freq_domain(incoming_angles=angles, signals=sounds)
    r = linear_array.add_white_noise(r)

# plot the last result as an example
linear_array.listen_pattern_fixed_input(r, sounds, labels, model=MicrophoneArray.w_mvdr)

# save audio for demos
output = linear_array.listen(v1_theta, r, model=MicrophoneArray.w_mvdr)
linear_array.load_write(output, f"audio_output/mvdr_v1_{n_voices}_{theta}.wav")
output = linear_array.listen(v2_theta, r, model=MicrophoneArray.w_mvdr)
linear_array.load_write(output, f"audio_output/mvdr_v2_{n_voices}_{theta}.wav")
output = linear_array.listen(v3_theta, r, model=MicrophoneArray.w_mvdr)
linear_array.load_write(output, f"audio_output/mvdr_v3_{n_voices}_{theta}.wav")
output = linear_array.listen(v4_theta, r, model=MicrophoneArray.w_mvdr)
linear_array.load_write(output, f"audio_output/mvdr_v4_{n_voices}_{theta}.wav")
output = linear_array.listen(m_theta, r, model=MicrophoneArray.w_mvdr)
linear_array.load_write(output, f"audio_output/mvdr_m1_{n_voices}_{theta}.wav")

# SCENARIO 2 EXAMPLE PLOT 2
theta = 5
v1_theta = np.deg2rad(theta/2) 
v2_theta = np.deg2rad(-theta/2) 
v3_theta = np.deg2rad(theta/2 + theta) 
v4_theta = np.deg2rad(-theta/2 - theta) 
m_theta = 0
angles = [v1_theta, v2_theta, v3_theta, v4_theta, m_theta]

results = np.zeros((2, len(angles)))
for each in np.arange(n_exp):
    # randomize experiment 
    # combine sounds based on acoustic model, outputs frequency domain signal at each microphone
    # add noise
    sounds = linear_array.randomize_experiment(n_voices,(len(angles)-n_voices))
    r = linear_array.combine_freq_domain(incoming_angles=angles, signals=sounds)
    r = linear_array.add_white_noise(r)

# plot the last result as an example
linear_array.listen_pattern_fixed_input(r, sounds, labels, model=MicrophoneArray.w_mvdr)

# save audio for demos
output = linear_array.listen(v1_theta, r, model=MicrophoneArray.w_mvdr)
linear_array.load_write(output, f"audio_output/mvdr_v1_{n_voices}_{theta}.wav")
output = linear_array.listen(v2_theta, r, model=MicrophoneArray.w_mvdr)
linear_array.load_write(output, f"audio_output/mvdr_v2_{n_voices}_{theta}.wav")
output = linear_array.listen(v3_theta, r, model=MicrophoneArray.w_mvdr)
linear_array.load_write(output, f"audio_output/mvdr_v3_{n_voices}_{theta}.wav")
output = linear_array.listen(v4_theta, r, model=MicrophoneArray.w_mvdr)
linear_array.load_write(output, f"audio_output/mvdr_v4_{n_voices}_{theta}.wav")
output = linear_array.listen(m_theta, r, model=MicrophoneArray.w_mvdr)
linear_array.load_write(output, f"audio_output/mvdr_m1_{n_voices}_{theta}.wav")


#%%
## SCENARIO 2 PLOT OVER THETA
aug_2 = []
sup_2 = []
aug_nv_2 = []
sup_nv_2 = []

for theta in theta_list:
    print(theta)
    v1_theta = np.deg2rad(theta/2) # direction of steering 
    v2_theta = np.deg2rad(-theta/2) # direction of incident sound 
    v3_theta = np.deg2rad(theta/2 + theta) # direction of steering 
    v4_theta = np.deg2rad(-theta/2 - theta) # direction of incident sound 
    m_theta = 0
    angles = [v1_theta, v2_theta, v3_theta, v4_theta, m_theta]

    results = np.zeros((2, len(angles)))
    for each in np.arange(n_exp):
        # randomize experiment 
        # combine sounds based on acoustic model, outputs frequency domain signal at each microphone
        # add noise
        sounds = linear_array.randomize_experiment(n_voices,(len(angles)-n_voices))
        r = linear_array.combine_freq_domain(incoming_angles=angles, signals=sounds)
        r = linear_array.add_white_noise(r)
        
        # record the percent improvmeent of eye-tracking
        results += linear_array.listen_improvement(r, sounds, angles, model=MicrophoneArray.w_mvdr)

        # if all plots are desired
        # linear_array.listen_pattern_fixed_input(r, sounds, labels, model=MicrophoneArray.w_mvdr)
    results = results/n_exp

    avg_augmentation = (results[0,0] + results[0,1])/2
    avg_supression = (results[1,0] + results[1,1])/2
    aug_2.append(avg_augmentation)
    sup_2.append(avg_supression)

    avg_augmentation = (results[0,2] + results[0,3])/2
    avg_supression = (results[1,2] + results[1,3])/2
    aug_nv_2.append(avg_augmentation)
    sup_nv_2.append(avg_supression)

# plot the percent improvment 
fig, ax = plt.subplots(1, 1, figsize=(7,3))
ax.set_title("Change in PSNR for Eye-Steered versus Head-Steered")
ax.set_ylabel("Percent (%) Change in PSNR")
ax.set_xlabel("Angular Distance Between Sources (deg)")
ax.plot(theta_list[:-1], aug_2[:-1], label='PSNR improvement in target source') 
ax.plot(theta_list[:-1], sup_2[:-1], label='PSNR suppression in interference source') 
ax.plot(theta_list[:-1], aug_nv_2[:-1], label='PSNR improvement in far source') 
ax.plot(theta_list[:-1], sup_nv_2[:-1], label='PSNR suppression in far interference source') 
ax.invert_xaxis()
ax.legend()
ax.set_ylim([0,1])
print(aug_2)
print(sup_2)
print(aug_nv_2)
print(sup_nv_2)
